@@ -26,10 +26,6 @@
 # include "mac.h"
 #endif
 
-#ifdef MSDOS_PROCS
-# include <process.h>
-#endif
-
 private void
 	DefAutoExec proto((const data_obj *(*proc) ptrproto((const char *))));
 
@@ -42,8 +38,8 @@ int	InJoverc = 0;
 private struct AutoExec {
 	const char	*a_pattern;
 	const data_obj	*a_cmd;
-	int	a_arg_state,
-		a_arg_count;
+	int	a_arg_state;
+	long	a_arg_count;
 } AutoExecs[NEXECS];	/* must be initialized by system to 0 */
 
 private int	ExecIndex = 0;
@@ -81,8 +77,10 @@ const data_obj	*(*proc) ptrproto((const char *));
 		&& p->a_arg_count == arg_count)
 			return;		/* eliminate duplicates */
 
-	if (ExecIndex >= NEXECS)
+	if (ExecIndex >= NEXECS) {
 		complain("Too many auto-executes, max %d.", NEXECS);
+		/* NOTREACHED */
+	}
 	p->a_pattern = copystr(pattern);
 	p->a_cmd = d;
 	p->a_arg_state = arg_state;
@@ -147,21 +145,21 @@ Extend()
 	ExecCmd(findcom(": "));
 }
 
-/* Read a positive integer from CP.  It must be in base BASE, and
+/* Read a positive long integer from CP.  It must be in base BASE, and
  * complains if it isn't.  If allints, all the characters
  * in the string must be integers or we return NO (failure); otherwise
  * we stop reading at the first nondigit and return YES (success).
  */
 bool
-chr_to_int(cp, base, allints, result)
+chr_to_long(cp, base, allints, result)
 register const char	*cp;
 int	base;
 bool	allints;
-register int	*result;
+register long	*result;
 {
 	register char	c;
-	int	value = 0,
-		sign;
+	long	value = 0;
+	int	sign;
 
 	if ((c = *cp) == '-') {
 		sign = -1;
@@ -176,14 +174,51 @@ register int	*result;
 			break;
 		}
 		c = c - '0';
-		if (c >= base)
+		if (c >= base) {
 			complain("You must specify in base %d.", base);
+			/* NOTREACHED */
+		}
 		value = value * base + c;
 	}
 	*result = value * sign;
 	return YES;
 }
 
+/* Read a positive integer from CP.  It must be in base BASE, and
+ * complains if it isn't.  If allints, all the characters
+ * in the string must be integers or we return NO (failure); otherwise
+ * we stop reading at the first nondigit and return YES (success).
+ */
+bool
+chr_to_int(cp, base, allints, result)
+register const char	*cp;
+int	base;
+bool	allints;
+register int	*result;
+{
+	long	value;
+	bool ret = chr_to_long(cp, base, allints, &value);
+	if (ret == YES)
+	    *result = (int)(value & ~0); /* XXX but no worse than before */
+	return ret;
+}
+
+long
+ask_long(def, prompt, base)
+const char	*def;
+const char	*prompt;
+int	base;
+{
+	const char	*val = ask(def, prompt);
+	long	value;
+
+	if (!chr_to_long(val, base, YES, &value)) {
+		complain("That's not a number!");
+		/* NOTREACHED */
+	}
+	return value;
+}
+
 int
 ask_int(def, prompt, base)
 const char	*def;
@@ -191,10 +226,12 @@ const char	*prompt;
 int	base;
 {
 	const char	*val = ask(def, prompt);
-	int	value;
+	int		value = 0;	/* avoid gcc complaint */
 
-	if (!chr_to_int(val, base, YES, &value))
+	if (!chr_to_int(val, base, YES, &value)) {
 		complain("That's not a number!");
+		/* NOTREACHED */
+	}
 	return value;
 }
 
@@ -255,6 +292,8 @@ vset_aux(vp, prompt)
 const struct variable	*vp;
 char	*prompt;
 {
+	if (vp->v_flags & V_READONLY)
+		complain("[cannot set readonly variable %s]", vp->Name);
 	switch (vp->v_flags & V_TYPEMASK) {
 	case V_INT:
 	case V_WHOLEX:
@@ -274,8 +313,10 @@ char	*prompt;
 
 		vpr_aux(vp, def, sizeof(def));
 		val = ask_int(def, prompt, (vp->v_flags & V_FMODE)? 8 : 10);
-		if (val < lwb)
+		if (val < lwb) {
 			complain("[%s must not be less than %d]", vp->Name, lwb);
+			/* NOTREACHED */
+		}
 		*((int *) vp->v_value) = val;
 		break;
 	    }
@@ -416,8 +457,10 @@ ZXchar	c;
 		int	i;
 		size_t	len = strlen(linebuf);
 
-		if (InJoverc)
+		if (InJoverc) {
 			complain("[invalid `?']");
+			/* NOTREACHED */
+		}
 		/* kludge: in case we're using UseBuffers, in which case
 		 * linebuf gets written all over (but restored by TOstop/TOabort)
 		 */
@@ -435,7 +478,7 @@ ZXchar	c;
 		/* let's do some completion! */
 		int
 			i,
-			numeral,
+			numeral = 0,		/* avoid gcc complaint */
 			len = strlen(linebuf),
 			minmatch = 1000,	/* init with dummy to placate picky compilers */
 			maxmatch = 0,
@@ -457,8 +500,10 @@ ZXchar	c;
 						comp_value = i;	/* good: done */
 						return NO;
 					} else {
-						if (InJoverc)
+						if (InJoverc) {
 							complain("[%s already exists]");
+							/* NOTREACHED */
+						}
 						add_mess(" [already exists]");
 						SitFor(7);
 						return YES;
@@ -489,9 +534,11 @@ ZXchar	c;
 			comp_value = lastmatch;
 			return NO;
 		}
-		if (InJoverc)
+		if (InJoverc) {
 			complain("[\"%s\" %s]", linebuf,
 				numfound == 0? "unknown" : "ambiguous");
+			/* NOTREACHED */
+		}
 		if (numfound == 0) {
 			/* Unknown: either not ALLOW_NEW (bad) or not CR (not good enough) */
 			add_mess(" [unknown]");
@@ -527,9 +574,10 @@ const char	*prompt;
 int	flags;
 {
 	/* protect static "Possible" etc. from being overwritten due to recursion */
-	if (InRealAsk)
+	if (InRealAsk) {
 		complain((char *) NULL);
-
+		/* NOTREACHED */
+	}
 	Possible = possible;
 	comp_flags = flags;
 	(void) do_ask("\r\n \t?", aux_complete, def, prompt);
@@ -554,9 +602,28 @@ Source()
 	if (!joverc(fnamebuf) && !silence) {
 		message(IOerr("read", fnamebuf));
 		complain((char *)NULL);
+		/* NOTREACHED */
 	}
 }
 
+/* TODO: Make this unsigned long when we dump support for pre-ANSI C */
+/* calculate percentage without float and no overflow */
+private int
+calc_percent(a, b)
+long	a, b;
+{
+	int v;
+	if (b == 0) {
+		v = 100;
+	} else if (a > (~(1UL << ((sizeof(long)*CHAR_BIT)-1)))/100) {
+		v = (int) (a / (b / 100));
+	} else {
+		v = (int) ((a * 100) / b);
+	}
+	return v;
+}
+	
+/* TODO: Make dotchar, nchars unsigned long when we dump support for pre-ANSI C */
 void
 BufPos()
 {
@@ -575,14 +642,9 @@ BufPos()
 		nchars += length(lp) + (lp->l_next != NULL);	/* include the NL */
 	}
 
-	/* Note: percent calculation might overflow if there are more
-	 * than 20 megs in a buffer (LONG_MAX may be as small as 2**31 - 1).
-	 * Does not seem worth fixing.  Using unsigned long would double
-	 * the limit, but unsigned long is not in K&R C.
-	 */
 	f_mess("[\"%s\" line %D/%D, char %D/%D (%d%%), cursor = %d/%d]",
 	       filename(curbuf), dotline, i, dotchar, nchars,
-	       (nchars == 0) ? 100 : (int) (((long) dotchar * 100) / nchars),
+	       calc_percent(dotchar, nchars),
 	       calc_pos(linebuf, curchar),
 	       calc_pos(linebuf, (int)strlen(linebuf)));
 	stickymsg = YES;
@@ -611,8 +673,10 @@ char	*cmd;
 			if (*ip == '\0')
 				break;
 
-			if (ap == &args[elemsof(args)])
+			if (ap == &args[elemsof(args)]) {
 				complain("Too many args for IF shell command");
+				/* NOTREACHED */
+			}
 			*ap++ = op;
 			for (;;) {
 				char
@@ -636,8 +700,10 @@ char	*cmd;
 						case '\\':
 							if (c == '"') {
 								c2 = *ip++;
-								if (c2 == '\0')
+								if (c2 == '\0') {
 									complain("Misplaced \\ in IF command");
+									/* NOTREACHED */
+								}
 							}
 							/*FALLTHROUGH*/
 						default:
@@ -648,8 +714,10 @@ char	*cmd;
 					continue;
 				case '\\':
 					c = *ip++;
-					if (c == '\0')
+					if (c == '\0') {
 						complain("Misplaced \\ in IF command");
+						/* NOTREACHED */
+					}
 					/*FALLTHROUGH*/
 				default:
 					*op++ = c;
@@ -681,10 +749,14 @@ char	*cmd;
 			/*NOTREACHED*/
 		}
 		dowait(&status);
-		if (!WIFEXITED(status))
+		if (!WIFEXITED(status)) {
 			complain("[no status returned from child in IF test]");
-		if (WIFSIGNALED(status))
+			/* NOTREACHED */
+		}
+		if (WIFSIGNALED(status)) {
 			complain("[IF test terminated by signal %d]", WTERMSIG(status));
+			/* NOTREACHED */
+		}
 		return WEXITSTATUS(status)==0;
 	}
 # else
@@ -692,8 +764,10 @@ char	*cmd;
 	{
 		int	status;
 
-		if ((status = spawnvp(0, args[0], args)) < 0)
+		if ((status = spawnvp(0, args[0], args)) < 0) {
 			complain("[Spawn failed: IF]");
+			/* NOTREACHED */
+		}
 		return (status == 0);	/* 0 means successful */
 	}
 #  else /* !MSDOS_PROCS */
@@ -712,8 +786,10 @@ char	*oppat;
 	int	len = strlen(verb);
 
 	if (caseeqn(inp, verb, (size_t)len) && LookingAt("[ \\t]\\|$", inp, len)) {
-		if (!LookingAt(oppat, inp, len))
+		if (!LookingAt(oppat, inp, len)) {
 			complain("[malformed %s]", verb);
+			/* NOTREACHED */
+		}
 		return YES;
 	}
 	return NO;
@@ -771,18 +847,23 @@ char	*file;
 			char	cmd[128];
 
 			finger <<= 1;
-			if (finger == 0)
+			if (finger == 0) {
 				complain("[`if' nested too deeply]");
-
+				/* NOTREACHED */
+			}
 			putmatch(1, cmd, sizeof cmd);
+			jdbg("if finger=0x%x skipping=0x%x inelse=0x%x cmd=\"%s\"\n", finger, skipping, inelse, cmd);
 			if (skipping == 0 && !do_if(cmd))
 				skipping |= finger;
 #endif /* SUBSHELL */
 #ifndef MAC	/* no environment in MacOS */
 		} else if (cmdmatch(Inputp, "ifenv", "\\>[ \t]*\\<\\([^ \t][^ \t]*\\)\\>[ \t]\\(.*\\)$")) {
 			finger <<= 1;
-			if (finger == 0)
+			if (finger == 0) {
 				complain("[`ifenv' nested too deeply]");
+				/* NOTREACHED */
+			}
+			jdbg("ifenv finger=0x%x skipping=0x%x inelse=0x%x\n", finger, skipping, inelse);
 			if (skipping == 0) {
 				char	envname[128],
 					envpat[128],
@@ -795,17 +876,45 @@ char	*file;
 					skipping |= finger;
 			}
 #endif
+		} else if (cmdmatch(Inputp, "ifvar", "\\>[ \t]*\\<\\([^ \t][^ \t]*\\)\\>[ \t]\\(.*\\)$")) {
+			finger <<= 1;
+			if (finger == 0) {
+				complain("[`if' nested too deeply]");
+				/* NOTREACHED */
+			}
+			jdbg("ifvar finger=0x%x skipping=0x%x inelse=0x%x\n", finger, skipping, inelse);
+			if (skipping == 0) {
+				char	vname[128],
+					vpat[128],
+					vbuf[128],
+					*val;
+				bool	matched = NO;
+
+				putmatch(1, vname, sizeof vname);
+				putmatch(2, vpat, sizeof vpat);
+				val = (char *)getvar(vname, vbuf, sizeof(vbuf));
+				jdbg("ifvar var \"%s\" => \"%s\" vpat \"%s\"\n", vname, val != NULL? val : "(null)", vpat);
+				if (val==NULL || !(matched = LookingAt(vpat, val, 0)))
+					skipping |= finger;
+				jdbg("ifvar matched=%d skipping=0x%x\n", matched, skipping);
+			}
 		} else if (cmdmatch(Inputp, "else", "[ \\t]*$")) {
-			if (finger == 1 || (inelse & finger))
+			if (finger == 1 || (inelse & finger)) {
 				complain("[Unexpected `else']");
+				/* NOTREACHED */
+			}
 			inelse |= finger;
 			skipping ^= finger;
+			jdbg("else finger=0x%x skipping=0x%x inelse 0x%x\n", finger, skipping, inelse);
 		} else if (cmdmatch(Inputp, "endif", "[ \\t]*$")) {
-			if (finger == 1)
+			if (finger == 1) {
 				complain("[Unexpected `endif']");
+				/* NOTREACHED */
+			}
 			inelse &= ~finger;
 			skipping &= ~finger;
 			finger >>= 1;
+			jdbg("endif finger=0x%x skipping=0x%x inlse=0x%x\n", finger, skipping, inelse);
 		} else if (skipping == 0) {
 			(void) strcat(Inputp, "\n");
 			for (;;) {
@@ -826,10 +935,12 @@ char	*file;
 
 				if ((c = peekchar) != EOF) {
 					peekchar = EOF;
-					if (Inputp != NULL && ZXRC(Inputp[-1]) == c)
+					if (Inputp != NULL && ZXRC(Inputp[-1]) == c) {
 						Inputp -= 1;
-					else if (!jiswhite(c) && c != '\n' && c != '\0')
+					} else if (!jiswhite(c) && c != '\n' && c != '\0') {
 						complain("[junk at end of line]");
+						/* NOTREACHED */
+					}
 				}
 
 				if (Inputp == NULL)
@@ -840,8 +951,10 @@ char	*file;
 				if (*Inputp == '\0' || *Inputp == '\n')
 					break;
 
-				if (this_cmd != ARG_CMD)
+				if (this_cmd != ARG_CMD) {
 					complain("[junk at end of line]");
+					/* NOTREACHED */
+				}
 			}
 		}
 	}
@@ -850,6 +963,7 @@ char	*file;
 	if (finger != 1) {
 		finger = 1;
 		complain("[Missing endif]");
+		/* NOTREACHED */
 	}
 #endif
 	f_close(fp);
