@@ -15,9 +15,7 @@
 #	include "mac.h"
 #else /* !MAC */
 #	include <sys/stat.h>
-#	ifdef MSFILESYSTEM
-#		include <io.h>
-#	else /* !MSFILESYSTEM */
+#	ifndef MSFILESYSTEM
 #		include <sys/file.h>
 #	endif /* !MSFILESYSTEM */
 #endif /* !MAC */
@@ -49,8 +47,10 @@ int	flags,
 		if (fp->f_flags == 0)
 			break;
 
-	if (i == MAXFILES)
+	if (i == MAXFILES) {
 		complain("[Too many open files!]");
+		/* NOTREACHED */
+	}
 	fp->f_bufsize = buf_size;
 	fp->f_cnt = 0;
 	fp->f_fd = fd;
@@ -166,6 +166,7 @@ File	*fp;
 		 * could easily print it.
 		 */
 		error("[%s error: %s]", what, strerror(err));
+		/* NOTREACHED */
 	}
 }
 
@@ -281,6 +282,7 @@ register File	*fp;
 					fp->f_flags |= F_ERR;
 					error("[I/O error(%s); file = %s, fd = %d]",
 						strerror(errno), fp->f_name, fp->f_fd);
+					/* NOTREACHED */
 #ifndef MSDOS
 				}
 #endif /* MSDOS */
@@ -319,8 +321,9 @@ size_t	max;
 			continue;
 
 		if (cp >= endp) {
+			fp->f_flags |= F_LINETOOLONG;
 			add_mess(" [Line too long]");
-			rbell();
+                        rbell();
 			return YES;
 		}
 		*cp++ = c;
