@@ -65,7 +65,7 @@ register LinePtr	after;
 
 void
 LineInsert(num)
-register int	num;
+register long	num;
 {
 	char	newline[LBSIZE];
 	register LinePtr	newdot,
@@ -445,7 +445,7 @@ int wrap_off;
 
 void
 open_lines(n)
-int	n;
+long	n;
 {
 	Bufpos	dot;
 
@@ -662,9 +662,9 @@ private bool
 newchunk()
 {
 	register LinePtr	newline;
-	register int	i;
+	register long	i;
 	ChunkPtr	f;
-	int	nlines = CHUNKSIZE;
+	long	nlines = CHUNKSIZE;
 	bool	done_gc = NO;
 
 	for (;;) {
@@ -706,8 +706,10 @@ nbufline()
 	if (faline == NULL) {
 		RecycleLines();
 		if (faline == NULL) {
-			if (!newchunk())
+			if (!newchunk()) {
 				complain("[Out of lines] ");
+				/* NOTREACHED */
+			}
 		}
 	}
 	newline = faline;
@@ -725,7 +727,7 @@ remfreelines(c)
 register ChunkPtr	c;
 {
 	register LinePtr	lp;
-	register int	i;
+	register long	i;
 
 	for (lp = c->c_block, i = c->c_nlines; i != 0 ; lp++, i--) {
 		if (lp->l_prev == NULL)
@@ -752,7 +754,7 @@ GCchunks()
 	register ChunkPtr	cp;
 	ChunkPtr	prev = NULL,
 			next;
-	register int	i;
+	register long	i;
 	register LinePtr	newline;
 
 	RecycleLines();
@@ -790,8 +792,10 @@ GSexpr()
 	Bufpos	dot,
 		end;
 
-	if (linebuf[curchar] != '(')
+	if (linebuf[curchar] != '(') {
 		complain((char *)NULL);
+		/* NOTREACHED */
+	}
 	DOTsave(&dot);
 	FSexpr();
 	DOTsave(&end);
