@@ -30,7 +30,6 @@
 #endif
 
 #ifdef MSFILESYSTEM
-# include <io.h>
 # include <direct.h>
 # include <dos.h>
 # include <stdlib.h>	/* _splitpath, _makepath */
@@ -62,7 +61,7 @@ private void
 #endif
 
 long	io_chars;		/* number of chars in this open_file */
-int	io_lines;		/* number of lines in this open_file */
+long	io_lines;		/* number of lines in this open_file */
 
 #ifdef BACKUPFILES
 bool	BkupOnWrite = NO;	/* VAR: make backup files when writing */
@@ -166,8 +165,10 @@ char *to;
 
 	if (to[1] == ':') {
 		d = CharUpcase(to[0]) - ('A' - 1);
-		if (d < 'A'-('A'-1) || 'Z'-('A'-1) < d)
+		if (d < 'A'-('A'-1) || 'Z'-('A'-1) < d) {
 			complain("invalid drive");
+			/* NOTREACHED */
+		}
 		_dos_getdrive(&dd);
 		if (dd != d)
 			_dos_setdrive(d, &dd);	/* ??? no failure report? */
@@ -301,6 +302,7 @@ const char	*complaint;
 	))
 	{
 		complain(complaint, buf);
+		/* NOTREACHED */
 	}
 	return fd;
 # endif /* !NO_MKTEMP */
@@ -337,7 +339,7 @@ File	*fp;
 {
 	if (fp != NULL) {
 		if (fp->f_flags & F_TELLALL)
-			add_mess(" %d lines, %D characters.",
+			add_mess(" %D lines, %D characters.",
 				 io_lines, io_chars);
 		f_close(fp);
 	}
@@ -416,6 +418,10 @@ bool	is_insert;
 {
 	Bufpos	save;
 	File	*fp;
+	bool	err;
+	int	save_type;
+	char	*tmpfname,
+		*save_fname;
 
 	if (!is_insert)
 		curbuf->b_ntbf = NO;
@@ -434,14 +440,49 @@ bool	is_insert;
 	}
 
 	DOTsave(&save);
+
+	/*
+	 * if read fails, e.g. too long line, it would be
+	 * dangerous to save the buffer back to the real
+	 * filename, so we mark the buffer as scratch (unless
+	 * we are inserting, in which case the buffer might
+	 * have other valuable info worth saving, even with
+	 * the truncated line, sigh!), and give it a temporary
+	 * fname, which we restore after the read succeeds.
+	 */
+	save_type = curbuf->b_type;
+	save_fname = curbuf->b_fname;
+	if (save_fname) {
+		tmpfname = (char *)emalloc(FILESIZE);
+		backup_name(save_fname, "+", tmpfname, FILESIZE);
+		jdbg("temporary name \"%s\"\n", tmpfname);
+		curbuf->b_fname = tmpfname;
+	}
+	if (!is_insert)
+		curbuf->b_type = B_SCRATCH;
+
 	dofread(fp);
+
 	if (is_insert && io_chars > 0) {
 		modify();
 		set_mark();
 	}
 	SetDot(&save);
 	getDOT();
+	err = (fp->f_flags & (F_ERR | F_LINETOOLONG)) != 0;
 	close_file(fp);
+	if (err) {
+		if (save_fname)
+			free((UnivPtr)save_fname);
+		complain("[Error reading file%s]", is_insert ? ", buffer has tmp name now" : ", buffer marked as scratch with tmp name");
+		/* NOTREACHED */
+	}
+	curbuf->b_type = save_type;
+	if (save_fname) {
+		jdbg("restoring original name \"%s\"\n", save_fname);
+		curbuf->b_fname = save_fname;
+		free((UnivPtr) tmpfname);
+	}
 }
 
 void
@@ -505,9 +546,10 @@ bool	okay_home;
 		size_t	PWDlen = strlen(PWD);
 		size_t	improvement = 0;
 
-		if (strlen(fname) >= (size_t)FILESIZE)
+		if (strlen(fname) >= (size_t)FILESIZE) {
 			complain("filename longer than %d", FILESIZE-1);
-
+			/* NOTREACHED */
+		}
 		strcpy(name_buf, fname);	/* default: unchanged */
 		if (strncmp(fname, PWD, PWDlen) == 0 && fname[PWDlen] == '/') {
 			/* Below current directory: strip prefix and /.
@@ -639,8 +681,10 @@ getCWD()
 		if (cwd == NULL || !chkCWD(cwd)) {
 #endif
 			cwd = getcwd(pathname, sizeof(pathname));
-			if (cwd == NULL)
+			if (cwd == NULL) {
 				error("Cannot get current directory");
+				/* NOTREACHED */
+			}
 #ifndef MAC	/* no environment in MacOS */
 		}
 	}
@@ -674,8 +718,10 @@ doPushd(newdir)
 {
 	UpdModLine = YES;
 	if (*newdir == '\0') {	/* Wants to swap top two entries */
-		if (list_next(DirStack) == NULL)
+		if (list_next(DirStack) == NULL) {
 			complain("pushd: no other directory.");
+			/* NOTREACHED */
+		}
 		newdir = dir_name(list_next(DirStack));
 		if (Dchdir(newdir) == -1)
 		{
@@ -719,11 +765,12 @@ Popd()
 {
 	char *newdir;
 
-	if (list_next(DirStack) == NULL)
+	if (list_next(DirStack) == NULL) {
 		complain("popd: directory stack is empty.");
+		/* NOTREACHED */
+	}
 	newdir = dir_name(list_next(DirStack));
-	if (Dchdir(newdir) == -1)
-	{
+	if (Dchdir(newdir) == -1) {
 		s_mess("popd: cannot change back into %s.", newdir);
 		return;
 	}
@@ -785,6 +832,7 @@ register char	*user,
 	add_mess(" [unknown user: %s]", user);
 	SitFor(7);
 	complain((char *)NULL);
+	/* NOTREACHED */
 }
 
 # endif /* USE_GETPWNAM */
@@ -849,8 +897,10 @@ char	*intobuf;
 			if (uendp == NULL)
 				uendp = name + strlen(name);
 			name += 1;
-			if ((size_t) (uendp - name) >= sizeof(unamebuf))
+			if ((size_t) (uendp - name) >= sizeof(unamebuf)) {
 				len_error(JMP_COMPLAIN);
+				/* NOTREACHED */
+			}
 			null_ncpy(unamebuf, name, (size_t) (uendp - name));
 			get_hdir(unamebuf, localbuf);
 			name = uendp;
@@ -1036,9 +1086,12 @@ JWriteFile()
 		register char	*cp = fnamebuf;
 		register char	c;
 
-		while ((c = *cp++) != '\0')
-			if (!jisprint(c) || strchr(badchars, c)!=NULL)
+		while ((c = *cp++) != '\0') {
+			if (!jisprint(c) || strchr(badchars, c)!=NULL) {
 				complain("'%p': bad character in filename.", c);
+				/* NOTREACHED */
+			}
+		}
 	}
 
 	filemunge(fnamebuf);
@@ -1117,6 +1170,7 @@ bool	complainifbad;
 			message(IOerr((F_MODE(how) == F_READ) ? "open" : "create",
 			    fname));
 			complain((char *)NULL);
+			/* NOTREACHED */
 		}
 	} else {
 		const char	*rd_only = NullStr;
@@ -1224,7 +1278,7 @@ JReadFile()
 		reloading;
 	Window
 		*wp;
-	int
+	long
 		curlineno;
 
 #ifdef MAC
@@ -1310,7 +1364,7 @@ tmpinit()
 #ifdef MAC
 		".joveXXX"	/* must match string in mac.c:Ffilter() */
 #else
-		"joveXXXXXX"
+		"jvXXXXXX"
 #endif
 		);
 	tfname = copystr(buf);
@@ -1515,33 +1569,33 @@ private Block
 
 private daddr	next_bno = 0;
 
-private void	(*blkio) ptrproto((Block *, SSIZE_T (*) ptrproto((int, UnivPtr, size_t))));
-
 /* Needed to comfort MS Visual C */
-private void real_blkio ptrproto((Block *, SSIZE_T (*) ptrproto((int, UnivPtr, size_t))));
+private void blkio proto((Block *, SSIZE_T (*) ptrproto((int, UnivPtr, size_t))));
 
 private void
-real_blkio(b, iofcn)
+blkio(b, iofcn)
 register Block	*b;
 register SSIZE_T	(*iofcn) ptrproto((int, UnivPtr, size_t));
 {
-	(void) lseek(tmpfd, (off_t)b->b_bno << JLGBUFSIZ, 0);
-	if ((*iofcn)(tmpfd, (UnivPtr) b->b_buf, (size_t)JBUFSIZ) != JBUFSIZ)
-		error("[Tmp file %s error: to continue editing would be dangerous]",
-			(iofcn == read) ? "READ" : "WRITE");
-}
+	off_t boff = bno_to_seek_off(b->b_bno);
+	SSIZE_T nb;
+	static bool first_time = YES;
 
-/* Needed to comfort MS Visual C */
-private void fake_blkio ptrproto((Block *, SSIZE_T (*) ptrproto((int, UnivPtr, size_t))));
-
-private void
-fake_blkio(b, iofcn)
-register Block	*b;
-register SSIZE_T	(*iofcn) ptrproto((int, UnivPtr, size_t));
-{
-	tmpinit();
-	blkio = real_blkio;
-	real_blkio(b, iofcn);
+	if (first_time) {
+		tmpinit();
+		first_time = NO;
+	}
+	if (lseek(tmpfd, boff, 0) < 0) {
+		error("[Tmp file seek error to %D: %d %s; to continue editing would be dangerous]",
+		      (long)boff, errno, strerror(errno));
+		/* NOTREACHED */
+	}
+	else if ((nb = (*iofcn)(tmpfd, (UnivPtr) b->b_buf, (size_t)JBUFSIZ)) != JBUFSIZ) {
+		error("[Tmp file %s error got %D: %d %s: to continue editing would be dangerous]",
+			(iofcn == read) ? "READ" : "WRITE", (long)nb,
+			nb < 0 ? errno : 0, nb < 0 ? strerror(errno): "");
+		/* NOTREACHED */
+	}
 }
 
 void
@@ -1551,11 +1605,16 @@ d_cache_init()
 			**hp;	/* Hash pointer */
 	register daddr	bno;
 
+	jdbg("MAX_BLOCKS=%D\n", (long)MAX_BLOCKS);
+	jdbg("CHNK_CHARS=%D\n", (long)CHNK_CHARS);
+	jdbg("BLK_CHNKS=%D\n", (long)BLK_CHNKS);
 #ifdef MALLOC_CACHE
 	if (b_cache == NULL) {
 		b_cache = (Block *) calloc((size_t)NBUF,sizeof(Block));
-		if (b_cache == NULL)
+		if (b_cache == NULL) {
 			error("cannot allocate buffer cache");
+			/* NOTREACHED */
+		}
 	}
 #endif /* MALLOC_CACHE */
 
@@ -1573,7 +1632,6 @@ d_cache_init()
 		bp->b_HASHnext = *(hp = &bht[B_HASH(bno)]);
 		*hp = bp;
 	}
-	blkio = fake_blkio;
 }
 
 void
@@ -1588,14 +1646,14 @@ SyncTmp()
 	 */
 	for (bno = 0; bno < next_bno; bno++) {
 		if ((b = lookup_block(bno)) != NULL && b->b_dirty) {
-			(*blkio)(b, (SSIZE_T (*) ptrproto((int, UnivPtr, size_t)))write);
+			blkio(b, (SSIZE_T (*) ptrproto((int, UnivPtr, size_t)))write);
 			b->b_dirty = NO;
 		}
 	}
 #else /* !MSDOS */
 	for (b = f_block; b != NULL; b = b->b_LRUnext)
 		if (b->b_dirty) {
-			(*blkio)(b, (SSIZE_T (*) ptrproto((int, UnivPtr, size_t)))write);
+			blkio(b, (SSIZE_T (*) ptrproto((int, UnivPtr, size_t)))write);
 			b->b_dirty = NO;
 		}
 #endif /* !MSDOS */
@@ -1660,7 +1718,7 @@ register Block	*bp;
 		bht[B_HASH(bp->b_bno)] = hp->b_HASHnext;
 
 	if (bp->b_dirty) {	/* do, now, the delayed write */
-		(*blkio)(bp, (SSIZE_T (*) ptrproto((int, UnivPtr, size_t)))write);
+		blkio(bp, (SSIZE_T (*) ptrproto((int, UnivPtr, size_t)))write);
 		bp->b_dirty = NO;
 	}
 
@@ -1697,8 +1755,10 @@ bool	IsWrite;
 	 * and we want to prevent space overflow from being undetected
 	 * through arithmetic overflow.
 	 */
-	if (bno >=  MAX_BLOCKS-1)
-		error("Tmp file too large.  Get help!");
+	if (bno >=  MAX_BLOCKS-1) {
+		error("Tmp file too large for line %D bno %D, max is %D.  Get help!", (long)atl, (long)bno, (long)(MAX_BLOCKS-1));
+		/* NOTREACHED */
+	}
 	nleft = JBUFSIZ - off;
 	if (lastb != NULL && lastb->b_bno == bno) {
 		bp = lastb;	/* same as last time */
@@ -1748,7 +1808,7 @@ bool	IsWrite;
 		 */
 
 		if (bno < next_bno)
-			(*blkio)(bp, read);
+			blkio(bp, read);
 		else
 			next_bno = bno + 1;
 	}
@@ -1777,6 +1837,22 @@ lsave()
 	DOLsave = NO;
 }
 
+/* build backup file name, also used by SetBuf */
+void
+backup_name(fname, btype, bfname, bfnamesize)
+const char	*fname,
+		*btype;
+char		*bfname;
+size_t		bfnamesize;
+{
+	char	*s = strrchr(fname, '/');
+	size_t	dirlen = (s == NULL)? 0 : s + 1 - fname;
+
+	jamstrsub(bfname, fname, bfnamesize);
+	swritef(bfname+dirlen, (size_t) (bfnamesize - dirlen), "#%s%s~",
+		fname+dirlen, btype);
+}
+
 #ifdef BACKUPFILES
 private void
 file_backup(fname)
@@ -1791,15 +1867,7 @@ char *fname;
 		buf[JBUFSIZ],
 		bfname[FILESIZE];
 
-	/* build backup file name */
-	{
-		char	*s = strrchr(fname, '/');
-		size_t	dirlen = (s == NULL)? 0 : s + 1 - fname;
-
-		strcpy(bfname, fname);
-		swritef(bfname+dirlen, (size_t) (sizeof(bfname) - dirlen), "#%s~",
-			fname+dirlen);
-	}
+	backup_name(fname, "", bfname, sizeof(bfname));
 
 	if ((ffd = open(fname, O_RDONLY | O_BINARY | O_CLOEXEC)) < 0)
 		return;	/* cannot open original file: nothing to backup, we assume */
@@ -1832,6 +1900,7 @@ char *fname;
 			(void) close(ffd);
 			complain("[cannot create backup \"%s\": %d %s]",
 				bfname, e, strerror(e));
+			/* NOTREACHED */
 		}
 	}
 
@@ -1848,25 +1917,30 @@ char *fname;
 				close(bffd);
 				close(ffd);
 				complain("[error writing backup: %d %s]", e, strerror(e));
+				/* NOTREACHED */
 			}
 			p += wr;
 			rr -= wr;
 		}
 	}
 
-	if (rr < 0 || close(ffd) != 0)
+	if (rr < 0 || close(ffd) != 0) {
 		complain("[error reading \"%s\": %d %s]", fname, errno, strerror(errno));
+		/* NOTREACHED */
+	}
 #  ifdef USE_FSYNC
 	if (fsync(bffd) != 0) {
 		int	e = errno;
 
 		(void) close(bffd);
 		complain("[error fsyncing backup: %d %s]", e, strerror(e));
+		/* NOTREACHED */
 	}
 #  endif /* USE_FSYNC */
-	if (close(bffd) != 0)
+	if (close(bffd) != 0) {
 		complain("[error closing backup: %d %s]", errno, strerror(errno));
-
+		/* NOTREACHED */
+	}
 # else /* MSFILESYSTEM */
 	/* This code is designed to fit withing the 8.3 limitation of
 	 * MSDOS ("FAT" -- huh!) file systems.  Even though newer versions
@@ -1889,8 +1963,10 @@ char *fname;
 	}
 	strcat(tmp, ".bak");
 	unlink(tmp);
-	if (rename(fname, tmp) != 0)
+	if (rename(fname, tmp) != 0) {
 		complain("[cannot rename to \"%s\": %s]", tmp, strerror(errno));
+		/* NOTREACHED */
+	}
 # endif /* MSFILESYSTEM */
 }
 #endif /* BACKUPFILES */
