@@ -58,7 +58,6 @@
 # include <bios.h>
 # include <dos.h>
 # include <time.h>
-# include <process.h>
 # define SIGHUP	99
 # ifdef OWCDOS
 #  include <malloc.h>	/* for _heapgrow */
@@ -102,6 +101,7 @@ SIGHANDLERTYPE	handler;
 #endif
 
 bool	TimeDisplayed = YES;	/* is time actually displayed in modeline? */
+char	JoveFeatures[MAXCOLS];	/* VAR: list of compiled-in features */
 
 #ifdef UNIX
 
@@ -176,19 +176,11 @@ int	global_maxfd;
 
 /* paths */
 
-/* path of machine-independent library */
-#ifdef SHAREDIR
+/* VAR: directory path of machine-independent library with joverc, docs, etc */
 char	ShareDir[FILESIZE] = SHAREDIR;
-#else
-char	ShareDir[FILESIZE];
-#endif
 
 /* VAR: directory/device to store tmp files */
-#ifdef TMPDIR
 char	TmpDir[FILESIZE] = TMPDIR;
-#else
-char	TmpDir[FILESIZE];
-#endif
 
 #ifdef SUBSHELL
 char
@@ -200,11 +192,11 @@ char
 # endif /* MSFILESYSTEM */
 #endif
 
-/* LibDir: path of machine-dependent library (for Portsrv and Recover) */
+/* VAR: directory path of machine-dependent library (for Portsrv and Recover) */
 
-#if defined(SUBSHELL) || defined(PIPEPROCS)
+#if defined(SUBSHELL) || defined(PIPEPROCS) || defined(RECOVER)
 # define NEED_LIBDIR	1
-private char	LibDir[FILESIZE] = LIBDIR;
+char	LibDir[FILESIZE] = LIBDIR;
 #endif
 
 /* finish: handle bad-news signals.
@@ -276,14 +268,8 @@ int	code;
 #ifdef UNIX
 	if (code != 0 && code != SIGHUP)
 		abort();
-# ifdef USE_EXIT
-	exit(0);
-# else
-	_exit(0);
-# endif
-#else /* !UNIX */
-	exit(0);
-#endif /* !UNIX */
+#endif /* UNIX */
+	EXIT(0);
 	/*NOTREACHED*/
 }
 
@@ -1043,8 +1029,10 @@ Push()
 	if (started)
 		kbd_strt();
 #  endif
-	if (forkerr != 0)
+	if (forkerr != 0) {
 		complain("[Fork failed: %s]", strerror(errno));
+		/* NOTREACHED */
+	}
 # endif /* !MSDOS_PROCS */
 }
 
@@ -1093,6 +1081,7 @@ getch()
 		 * we're reading a .joverc file.
 		 */
 		complain("[command line too short]");
+		/* NOTREACHED */
 	}
 
 #ifdef RECOVER
@@ -1125,26 +1114,6 @@ getch()
 	return LastKeyStruck = c;
 }
 
-#if defined(SUBSHELL) && defined(RECOVER)
-private void
-dorecover()
-{
-	char	Recover[FILESIZE];	/* path to recover program (in LibDir) */
-
-	/* Since recover is a normal cooked mode program, reset the terminal */
-	UnsetTerm(NO);
-# ifdef PIPEPROCS
-	kbd_kill();		/* kill the keyboard process */
-# endif
-	PathCat(Recover, sizeof(Recover), LibDir, "recover");
-	execl(Recover, "recover", "-d", TmpDir, (char *) NULL);
-	writef("%s: execl failed! %s\n", Recover, strerror(errno));
-	flushscreen();
-	_exit(-1);
-	/* NOTREACHED */
-}
-#endif /* defined(SUBSHELL) && defined(RECOVER) */
-
 void
 ShowVersion()
 {
@@ -1156,15 +1125,15 @@ UNIX_cmdline(argc, argv)
 int	argc;
 char	*argv[];
 {
-	int	lineno = 0,
-		nwinds = 1;
+	long	lineno = 0;
+	int	nwinds = 1;
 	char	*pattern = NULL;
 
 	while (argc > 1) {
 		switch (argv[1][0]) {
 		case '+':
 			if ('0' <= argv[1][1] && argv[1][1] <= '9') {
-				(void) chr_to_int(&argv[1][1], 10, NO, &lineno);
+				(void) chr_to_long(&argv[1][1], 10, NO, &lineno);
 				break;
 			} else switch (argv[1][1]) {
 			case '\0':
@@ -1184,7 +1153,7 @@ char	*argv[];
 				break;
 			default:
 				error("Invalid switch %s",argv[1]);
-				break;
+				/* NOTREACHED */
 			}
 			break;
 		case '-':
@@ -1234,7 +1203,7 @@ char	*argv[];
 				break;
 			default:
 				error("Invalid switch %s",argv[1]);
-				break;
+				/* NOTREACHED */
 			}
 			break;
 		default:
@@ -1259,10 +1228,12 @@ char	*argv[];
 					Bufpos	*bufp;
 
 					if ((bufp = dosearch(pattern, FORWARD, UseRE)) != NULL
-					|| (bufp = dosearch(pattern, BACKWARD, UseRE)) != NULL)
+					|| (bufp = dosearch(pattern, BACKWARD, UseRE)) != NULL) {
 						SetDot(bufp);
-					else
+					} else {
 						complain("Couldn't match pattern in file.");
+						/* NOTREACHED */
+					}
 				}
 				if (nwinds > 0) {
 					nwinds -= 1;
@@ -1313,6 +1284,7 @@ error(fmt, va_alist)
 	}
 	rbell();
 	longjmp(mainjmp, JMP_ERROR);
+	/* NOTREACHED */
 }
 
 #ifdef STDARGS
@@ -1335,6 +1307,7 @@ complain(fmt, va_alist)
 	}
 	rbell();
 	longjmp(mainjmp, JMP_COMPLAIN);
+	/* NOTREACHED */
 }
 
 /* format and display a message without using the normal display mechanisms */
@@ -1382,8 +1355,10 @@ confirm(fmt, va_alist)
 	va_init(ap, fmt);
 	format(mesgbuf, sizeof mesgbuf, fmt, ap);
 	va_end(ap);
-	if (!yes_or_no_p("%s", mesgbuf))
+	if (!yes_or_no_p("%s", mesgbuf)) {
 		longjmp(mainjmp, JMP_COMPLAIN);
+		/* NOTREACHED */
+	}
 }
 
 /* Recursive edit.
@@ -1406,8 +1381,10 @@ Recur()
 	DoKeys(NO);	/* NO means not first time */
 	UpdModLine = YES;
 	RecDepth -= 1;
-	if (!valid_bp(b))
+	if (!valid_bp(b)) {
 		complain("Buffer gone!");
+		/* NOTREACHED */
+	}
 	tiewind(curwind, b);
 	SetBuf(b);
 	if (!is_an_arg())
@@ -1586,6 +1563,120 @@ bool	dousr;
 	}
 }
 
+private void
+setfeatures()
+{
+	size_t jlen = 0;
+	JoveFeatures[0] = '\0';
+#ifdef UNIX
+	jamstrsub(JoveFeatures + jlen, ":unix", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef MSDOS
+	jamstrsub(JoveFeatures + jlen, ":msdos", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef WIN32
+	jamstrsub(JoveFeatures + jlen, ":win32", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef MAC
+	jamstrsub(JoveFeatures + jlen, ":mac", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef IBMPCDOS
+	jamstrsub(JoveFeatures + jlen, ":ibmpc", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef ABBREV
+	jamstrsub(JoveFeatures + jlen, ":abbr", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef BACKUPFILES
+	jamstrsub(JoveFeatures + jlen, ":bak", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef BIFF
+	jamstrsub(JoveFeatures + jlen, ":biff", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef CMT_FMT
+	jamstrsub(JoveFeatures + jlen, ":cmtfmt", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef F_COMPLETION
+	jamstrsub(JoveFeatures + jlen, ":fcomp", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef IPROCS
+	jamstrsub(JoveFeatures + jlen, ":iproc", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef PIPEPROCS
+	jamstrsub(JoveFeatures + jlen, ":pipe", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef PTYPROCS
+	jamstrsub(JoveFeatures + jlen, ":pty", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef LISP
+	jamstrsub(JoveFeatures + jlen, ":lisp", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef SUBSHELL
+	jamstrsub(JoveFeatures + jlen, ":proc", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef SPELL
+	jamstrsub(JoveFeatures + jlen, ":spell", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef RECOVER
+	jamstrsub(JoveFeatures + jlen, ":rec", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef JOB_CONTROL
+	jamstrsub(JoveFeatures + jlen, ":job", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef JSMALL
+	jamstrsub(JoveFeatures + jlen, ":jsmall", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef ID_CHAR
+	jamstrsub(JoveFeatures + jlen, ":idchar", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef HIGHLIGHTING
+	jamstrsub(JoveFeatures + jlen, ":hl", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef JTC
+	jamstrsub(JoveFeatures + jlen, ":jtc", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef TERMCAP
+	jamstrsub(JoveFeatures + jlen, ":tcap", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef TERMINFO
+	jamstrsub(JoveFeatures + jlen, ":tinfo", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef USE_CTYPE
+	jamstrsub(JoveFeatures + jlen, ":ctype", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+#ifdef ISO_8859_1
+	jamstrsub(JoveFeatures + jlen, ":iso88591", sizeof(JoveFeatures) - jlen);
+	jlen = strlen(JoveFeatures);
+#endif
+	jamstrsub(JoveFeatures + jlen, ":", sizeof(JoveFeatures) - jlen);
+	jdbg("jove-features=\"%s\"\n", JoveFeatures);
+}
+
+
 int
 main(argc, argv)
 int	argc;
@@ -1620,6 +1711,16 @@ char	*argv[];
 	iniargc = argc;
 	iniargv = argv;
 
+	if ((argp = scanvec(argv, "-D")) != NULL) {
+		jdpath = argp[1];
+		jdbg("jove debug started %s\n",
+		     get_time((time_t *)NULL, (char *)NULL, 0, -1));
+	}
+	jdbg("MAXCOLS=%d\n", MAXCOLS);
+	jdbg("NBUF=%d\n", NBUF);
+	jdbg("JBUFSIZ=%d\n", JBUFSIZ);
+	jdbg("NCHARS=%d\n", NCHARS);
+
 	if (setjmp(mainjmp)) {
 		ttysetattr(NO);
 		writef("\nAck! I can't deal with error \"%s\" now.\n", mesgbuf);
@@ -1632,11 +1733,94 @@ char	*argv[];
 	locale_adjust();
 #endif
 
-	getTERM();	/* Get terminal. */
 #ifndef MAC	/* no environment in MacOS */
 	if (getenv("METAKEY"))
 		MetaKey = YES;
-#endif
+
+	/* Handle overrides for ShareDir and LibDir.
+	 * We take care to use the last specification.
+	 * Even if we don't use LibDir, we accept it.
+	 */
+	 {
+		char
+			*so = getenv("JOVESHARE");
+# ifdef NEED_LIBDIR
+		char
+			*lo = getenv("JOVELIB");
+# endif
+
+		for (argp = argv; argp[0] != NULL && argp[1] != NULL; argp++) {
+			if (strcmp(*argp, "-s") == 0)
+				so = *++argp;
+# ifdef NEED_LIBDIR
+			else if (strcmp(*argp, "-l") == 0)
+				lo = *++argp;
+			else if (strcmp(*argp, "-ls") == 0 || strcmp(*argp, "-sl") == 0)
+				lo = so = *++argp;
+# endif
+		}
+		if (so != NULL)
+			if (!carefulcpy(ShareDir, so, sizeof(ShareDir)-9, "ShareDir", YES))
+				finish(0);
+# ifdef NEED_LIBDIR
+		if (lo != NULL)
+			if (!carefulcpy(LibDir, lo, sizeof(LibDir)-9, "LibDir", YES))
+				finish(0);
+#  ifdef PIPEPROCS
+		PathCat(Portsrv, sizeof(Portsrv), LibDir, "portsrv");
+#  endif
+# endif /* NEED_LIBDIR */
+	}
+
+	/* import the temporary file path from the environment
+	 * and fix the string, so that we can append a slash
+	 * safely
+	 */
+# ifdef MSFILESYSTEM
+	envcpy(TmpDir, "TEMP");
+# else
+	envcpy(TmpDir, "TMPDIR");
+# endif
+	{
+		char	*cp = &TmpDir[strlen(TmpDir)];
+
+		do {} while (cp != TmpDir && (*--cp == '/'
+# ifdef MSFILESYSTEM
+			|| *cp == '\\'
+# endif
+			));
+		cp[1] = '\0';
+	}
+
+# ifdef SUBSHELL
+#  ifdef MSFILESYSTEM	/* ??? Is this the right test? */
+	envcpy(Shell, "COMSPEC");
+	/* SHELL, if present in DOS environment, will take precedence over COMSPEC */
+#  endif /* MSFILESYSTEM */
+	envcpy(Shell, "SHELL");
+#  ifdef RECOVER
+	if (scanvec(argv, "-r") != NULL) {
+		char	Recover[FILESIZE];	/* path to recover program (in LibDir) */
+
+		PathCat(Recover, sizeof(Recover), LibDir, "recover");
+#  ifdef MSFILESYSTEM
+		jamstrcat(Recover, ".exe", sizeof(Recover));
+		if (spawnl(P_WAIT, Recover, "recover", "-d", TmpDir, (char *)NULL) == -1)
+#  else /* !MSFILESYSTEM */
+		if (execl(Recover, "recover", "-d", TmpDir, (char *) NULL) == -1)
+#  endif /* !MSFILESYSTEM */
+		{
+			writef("%s: execl failed! %s\n", Recover, strerror(errno));
+			EXIT(-1);
+		}
+		EXIT(0); /* only Win32, but no harm otherwise, avoids another ifdef */
+		/* NOTREACHED */
+	}
+#  endif /* RECOVER */
+# endif /* SUBSHELL */
+#endif /* ! MAC */
+
+	getTERM();	/* Get terminal. */
 	ttysetattr(YES);
 	ttsize();
 
@@ -1673,12 +1857,6 @@ char	*argv[];
 	else
 		getCWD();	/* After we setup curbuf in case we have to getwd() */
 
-	if ((argp = scanvec(argv, "-D")) != NULL) {
-		jdpath = argp[1];
-		jdbg("jove debug started %s\n",
-		     get_time((time_t *)NULL, (char *)NULL, 0, -1));
-	}
-
 #ifdef MAC
 	HomeDir = gethome();
 #else /* !MAC */
@@ -1713,86 +1891,16 @@ char	*argv[];
 
 	settout();	/* not until we know baudrate */
 	SetTerm();
-
-#ifndef MAC	/* no environment in MacOS */
-	/* Handle overrides for ShareDir and LibDir.
-	 * We take care to use the last specification.
-	 * Even if we don't use LibDir, we accept it.
-	 */
-	 {
-		char
-			*so = getenv("JOVESHARE");
-# ifdef NEED_LIBDIR
-		char
-			*lo = getenv("JOVELIB");
-# endif
-
-		for (argp = argv; argp[0] != NULL && argp[1] != NULL; argp++) {
-			if (strcmp(*argp, "-s") == 0)
-				so = *++argp;
-# ifdef NEED_LIBDIR
-			else if (strcmp(*argp, "-l") == 0)
-				lo = *++argp;
-			else if (strcmp(*argp, "-ls") == 0 || strcmp(*argp, "-sl") == 0)
-				lo = so = *++argp;
-# endif
-		}
-		if (so != NULL)
-			if (!carefulcpy(ShareDir, so, sizeof(ShareDir)-9, "ShareDir", YES))
-				finish(0);
-# ifdef NEED_LIBDIR
-		if (lo != NULL)
-			if (!carefulcpy(LibDir, lo, sizeof(LibDir)-9, "LibDir", YES))
-				finish(0);
-#  ifdef PIPEPROCS
-		PathCat(Portsrv, sizeof(Portsrv), LibDir, "portsrv");
-#  endif
-# endif /* NEED_LIBDIR */
-	}
-#endif /* !MAC */
+	setfeatures();
 
 	ShowVersion();	/* but the 'carefulcpy's which follow might overwrite it */
 
-	/* import the temporary file path from the environment
-	 * and fix the string, so that we can append a slash
-	 * safely
-	 */
-#ifdef MSFILESYSTEM
-	envcpy(TmpDir, "TEMP");
-#endif
-#ifndef MAC	/* no environment in MacOS */
-	envcpy(TmpDir, "TMPDIR");
-#endif
-	{
-		char	*cp = &TmpDir[strlen(TmpDir)];
-
-		do {} while (cp != TmpDir && (*--cp == '/'
-#ifdef MSFILESYSTEM
-			|| *cp == '\\'
-#endif
-			));
-		cp[1] = '\0';
-	}
-
-#ifdef SUBSHELL
-# ifdef MSFILESYSTEM	/* ??? Is this the right test? */
-	envcpy(Shell, "COMSPEC");
-	/* SHELL, if present in DOS environment, will take precedence over COMSPEC */
-# endif /* MSFILESYSTEM */
-	envcpy(Shell, "SHELL");
-#endif /* SUBSHELL */
-
 #ifdef UNIX
 	envcpy(Mailbox, "MAIL");
 #endif
 
 	dojovercs(scanvec(argv, "-J") == NULL, scanvec(argv, "-j") == NULL);
 
-#if defined(SUBSHELL) && defined(RECOVER)
-	if (scanvec(argv, "-r") != NULL)
-		dorecover();
-#endif
-
 #ifdef UNIX
 	/*
 	 * Jove binds INTR to a key, typically ^], which can
