@@ -123,7 +123,7 @@
 # define UNMACRO(proc)	(proc)
 #endif
 
-/* Since we don't use stdio.h, we may have to define NULL and EOF */
+/* Since Jove does not use stdio.h (recover and setmaps do), we may have to define NULL and EOF */
 
 #ifndef NULL
 #define NULL	(void *)0
@@ -133,6 +133,19 @@
 #define EOF	(-1)
 #endif
 
+/* Pervasive data types and constants */
+
+#ifndef CHAR_BIT
+# define CHAR_BIT	8	/* factor to convert sizeof => bits */
+#endif
+
+#ifndef NCHARS
+# define NCHARS 0400
+#endif
+
+#define NULL_DADDR	((daddr) 0)
+#define DDIRTY		((daddr)1 << (sizeof(daddr)*CHAR_BIT - 1))	/* daddr dirty flag */
+
 #define private		static
 
 typedef int	bool;
@@ -176,7 +189,7 @@ extern SIGHANDLERTYPE setsighandler proto((int, SIGHANDLERTYPE));
   * signal handlers need to be re-established.
   */
 # define resetsighandler(signo, handler)	{ \
-	static SIGHANDLERTYPE   reset_handler = NULL; \
+	static SIGHANDLERTYPE	reset_handler; \
  \
 	if (reset_handler != (handler)) \
 		reset_handler = setsighandler((signo), (handler)); \
@@ -280,8 +293,6 @@ extern volatile bool	UpdModLine;	/* Does the mode line need to be updated? */
 
 /* term.c: universal termcap-like declarations */
 
-#define MAXCOLS		512	/* maximum number of columns */
-
 extern int
 	SG,		/* number of magic cookies left by SO and SE */
 	LI,		/* number of lines */
@@ -321,8 +332,9 @@ typedef struct FileStruct	File;	/* fp.h */
 /* paths */
 
 extern char
-	ShareDir[FILESIZE],	/* path of machine-independent library */
-	TmpDir[FILESIZE];	/* VAR: directory/device to store tmp files */
+	ShareDir[FILESIZE],	/* VAR: directory path of machine-independent library */
+	LibDir[FILESIZE],	/* VAR: directory path of machine-dependenty libray */
+	TmpDir[FILESIZE];	/* VAR: directory path to store tmp files */
 
 #ifdef SUBSHELL
 extern char
@@ -391,7 +403,7 @@ extern ZXchar
 extern void
 	cmd_sync proto((void)),
 	add_stroke proto((ZXchar)),
-	error proto((const char *, ...)),
+	error proto((const char *, ...)) NEVER_RETURNS,
 	complain proto((const char *, ...)) NEVER_RETURNS,
 	raw_complain proto((const char *, ...)),
 	confirm proto((const char *, ...)),
@@ -416,7 +428,8 @@ extern void
 /* Variables: */
 
 extern bool	MetaKey;		/* VAR: this terminal has a meta key */
-extern bool	TimeDisplayed;	/* is time actually displayed in modeline? */
+extern bool	TimeDisplayed;		/* is time actually displayed in modeline? */
+extern char	JoveFeatures[MAXCOLS];	/* VAR: list of compiled-in features */
 #ifdef UNIX
 extern int	UpdFreq;		/* VAR: how often to update modeline */
 extern void	SetClockAlarm proto((bool unset));
