@@ -80,7 +80,7 @@ make_scr()
 	ns = Screen = (struct screenline *)
 			malloc((unsigned) LI * sizeof(struct screenline));
 
-	nsp = screenchars = (char *) malloc((unsigned)CO * LI);
+	nsp = screenchars = (char *) malloc((size_t)CO * LI);
 
 	if (DesiredScreen == NULL
 	|| PhysScreen == NULL
@@ -414,7 +414,7 @@ bool	abortable;
 	 * to ensure Placur does not get into trouble.  (UGLY!)
 	 */
 	struct LErange	*oldhl = Curline->s_effects;
-	static const struct LErange	nohl = { 0, 0, NULL, NULL };
+	static const struct LErange	nohl = { 0, 0, LENULLPROC, LENULLPROC };
 
 	if (oldhl != NOEFFECT) {
 		int	w = Curline->s_roof - Curline->s_line;
@@ -422,7 +422,7 @@ bool	abortable;
 		if (oldhl->norm != NULL)
 			MinCol = w;
 		if (oldhl->high != NULL && w > (int)oldhl->start)
-			MinCol = max(MinCol, min(w, (int) (oldhl->start + oldhl->width)));
+			MinCol = jmax(MinCol, jmin(w, (int) (oldhl->start + oldhl->width)));
 	}
 	/* If either the old line or the new line has effects,
 	 * we know that some effects processing is necessary.
@@ -432,13 +432,13 @@ bool	abortable;
 	 */
 	if (hl != NOEFFECT) {
 		if (hl->high != NULL)
-			MinCol = max(MinCol, (int) (hl->start + hl->width));
+			MinCol = jmax(MinCol, (int) (hl->start + hl->width));
 		if (oldhl == NOEFFECT) {
 			oldhl = Curline->s_effects = LEnew();	/* keep Placur on-track */
 			*oldhl = nohl;
 		}
 	}
-	real_effect = NULL;
+	real_effect = LENULLPROC;
 #else /* !HIGHLIGHTING */
 	if (Curline->s_effects != hl)
 		MinCol = Curline->s_roof - Curline->s_line;	/* must obliterate old */
